# Aether Framework

> A fast and flexible REST API framework built using TypeScript decorators.

The Aether framework is a simplistic RESTful API framework built on top of Express, and TypeScript, you will need to use TypeScript to utilise the functionality. The framework is built around my immediate issue and was for prototyping applications with very little resistance as the framework provides a series of decorators and a server.

_**NOTE**: Please note this framework is for my own learning and is heavily based around my personal coding style, if you have better ideas of how something could be done better, then please, let me know so I can update it or add new ideas._

---

## Features

### Implemented

Below lists features that are currently implemented.

| Feature | Description |
| - | - |
| Web Server | The implementation is a standard Express HTTP server. |
| Middleware | This relates to implementing a function for applying Express middleware functions. |
| Controller Decorator | This decorator implements a prefixed path for all child methods. |
| Route Decorator | Defines the HTTP method and the path (after the prefix) for that method. |
| Inject Decorator | The inject method will allow you to inject provided classes, these can be helpers etc. |
| Provide Decorator | The provide decorator will allow other methods to inject and resolve the provided class. |

---

### Future

Below lists features that will be implemented in time.

| Feature | Description |
| - | - |
| Threaded Decorator | Defines whether a method should be called in a separate thread because it may be CPU intensive and could slow the application. |
| Access Decorator | A decorator and built in permission system based on a hierarchical node system, that will be built in to the platform. |
| Authenticated Decorator | The decorator defines that a user should be authenticated, passing an optional role, will check against that user's role.
| Model Decorators | This will allow you to define models that can be used for data mapping, using our built in ORM. |

---

## Decorators

### @Controller(path: string)

**STATUS**: Beta (Stable, API Won't Change).

The controller decorator is used to define a controller and will use the path to generate a _prefix_ path for all routes inside.

```typescript
import { Controller } from 'aether-framework';

@Controller('/test')
class TestController {
	// Methods will be added here.
}
```

The above defines that all child routes will be prefixed with /test.

---

### @Route(method: string, path: string)

**STATUS**: Beta (Stable, API Won't Change).

The route method defines a route around a controller method, the method parameter is either the HTTP method: POST, GET, PUT, etc. The path parameter is the last part of the path (after the prefix) to that method.

```typescript
import { Controller, Route, IReturn, IContext } from 'aether-framework';

@Controller('/test')
class TestController {

	@Route('GET', '/foo')
	public async foo(context: IContext): Promise<IReturn> {
		// Method logic will go here.
	}
}
```

So if we say that our server is running on port 8080, then the path to that method would be: `http://localhost:8080/test/foo`.

---

### @Provide(uniqueName: string)

**STATUS**: Alpha (Not Stable, API Could Change).

The provide decorator will accept a unique name for the class you wish to provide and then save it ready for it to be called from any other application. Use the Inject decorator to inject a provided class into an application.

```typescript
import { Provide } from 'aether-framework';

@Provide('test_helper')
class TestHelper {

	public function say(message: string): void {
		console.log(message);
	}
}
```
---

### @Inject(uniqueName: string | string[])

**STATUS**: Alpha (Not Stable, API Could Change).

Once a class is provided using the decorator, then you can use the Inject decorator to inject into a class, now when using inject, you use it against a method, but will enable it for all methods inside, this is a bug I am trying to figure out, and is my own take on it.

```typescript
import { Controller, Route, IReturn, IContext, Inject, Provide } from 'aether-framework';

@Provide('test_helper')
class TestHelper {

	public function say(message: string): void {
		console.log(message);
	}
}

@Controller('/test')
class TestController {

	@Route('GET', '/foo')
	@Inject('test_helper')
	public async foo(context: IContext): Promise<IReturn> {
		// Method logic will go here.
	}
}
```